class WaterMeter:
    """Represents the water meter."""

    def __init__(self, volume: int, duration: int) -> None:
        """Create an Water Meter object."""

        self.volume = volume
        self.duration = duration


import aiohttp
import pytest
from aioresponses import aioresponses
from watergate_local_api import WatergateLocalApiClient, WatergateApiException
from watergate_local_api.models import DeviceState, NetworkingData, TelemetryData, AutoShutOffReport

@pytest.fixture
def client():
    # Return an uninitialized instance of the client
    return WatergateLocalApiClient(base_url="http://testserver")

@pytest.mark.asyncio
async def test_get_device_state(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic", payload={
            "valveState": "closed",
            "waterFlowing": False,
            "mqttConnected": True,
            "wifiConnected": True,
            "powerSupply": "external",
            "firmwareVersion": "2024.1.0",
            "waterMeter": {"volume": 567820, "duration": 3908},
            "uptime": 1024560
        })

        device_state = await client.async_get_device_state()
        assert device_state.valve_state == "closed"
        assert device_state.water_flow_indicator is False
        assert device_state.mqtt_status is True
        assert device_state.wifi_status is True
        assert device_state.power_supply == "external"
        assert device_state.firmware_version == "2024.1.0"
        assert device_state.uptime == 1024560
        assert device_state.water_meter.volume == 567820

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_get_networking(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic/networking", payload={
            "mqttConnected": True,
            "wifiConnected": True,
            "ip": "192.168.21.37",
            "gateway": "192.168.1.0",
            "subnet": "192.168.1.0/24",
            "ssid": "MyWiFi",
            "rssi": -45,
            "wifiUpTime": 1024560,
            "mqttUpTime": 1023450
        })

        networking_data = await client.async_get_networking()
        assert networking_data.ip == "192.168.21.37"
        assert networking_data.gateway == "192.168.1.0"
        assert networking_data.subnet == "192.168.1.0/24"
        assert networking_data.ssid == "MyWiFi"
        assert networking_data.rssi == -45

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_get_telemetry_data(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic/telemetry", payload={
            "flow": 6800,
            "pressure": 2320,
            "temperature": 23.5,
            "event": {"volume": 16000, "duration": 90},
            "errors": ["flow"]
        })

        telemetry_data = await client.async_get_telemetry_data()
        assert telemetry_data.flow == 6800
        assert telemetry_data.pressure == 2320
        assert telemetry_data.water_temperature == 23.5
        assert telemetry_data.ongoing_event.volume == 16000
        assert telemetry_data.ongoing_event.duration == 90
        assert "flow" in telemetry_data.errors

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_patch_auto_shut_off(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.patch("http://testserver/api/sonic/auto-shut-off", status=204)

        result = await client.async_patch_auto_shut_off(enabled=True, duration=10, volume=5)
        assert result is True

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_get_auto_shut_off_report(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic/auto-shut-off/report", payload={
            "type": "VOLUME_THRESHOLD",
            "volume": 300,
            "duration": 60,
            "timestamp": 1623456789
        })

        report = await client.async_get_auto_shut_off_report()
        assert report.type == "VOLUME_THRESHOLD"
        assert report.volume == 300
        assert report.duration == 60
        assert report.timestamp == 1623456789

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_set_webhook_url(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.patch("http://testserver/api/sonic/webhook", status=204)

        result = await client.async_set_webhook_url("http://webhook.url")
        assert result is True

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_retry_logic(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic", status=500)

        with pytest.raises(WatergateApiException):
            await client.async_get_device_state()

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_custom_exception(client):
    await client.__aenter__()

    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic", exception=aiohttp.ClientError)

        with pytest.raises(WatergateApiException):
            await client.async_get_device_state()

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_api_rate_limit_handling(client):
    await client.__aenter__()

    # Simulate API rate limit response (HTTP 429)
    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic", status=429)

        with pytest.raises(WatergateApiException):
            await client.async_get_device_state()

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_get_networking_with_unexpected_status_code(client):
    await client.__aenter__()

    # Simulate an unexpected status code (HTTP 403)
    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic/networking", status=403)

        with pytest.raises(WatergateApiException):
            await client.async_get_networking()

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_set_webhook_url_invalid_response(client):
    await client.__aenter__()

    # Simulate an invalid response when setting webhook (e.g., status 400)
    with aioresponses() as mock:
        mock.patch("http://testserver/api/sonic/webhook", status=400)

        with pytest.raises(WatergateApiException):
            await client.async_set_webhook_url("http://invalid-webhook.url")

    await client.__aexit__(None, None, None)

@pytest.mark.asyncio
async def test_auto_shut_off_report_with_missing_fields(client):
    await client.__aenter__()

    # Response with some fields missing
    with aioresponses() as mock:
        mock.get("http://testserver/api/sonic/auto-shut-off/report", payload={"type": "VOLUME_THRESHOLD"})

        report = await client.async_get_auto_shut_off_report()
        assert report.type == "VOLUME_THRESHOLD"
        assert report.volume is None  # Missing fields should default to None

    await client.__aexit__(None, None, None)


TIMESTAMP_FIELD = "timestamp"
TYPE_FIELD = "type"
DURATION_FIELD = "duration"
VOLUME_FIELD = "volume"

class AutoShutOffReport:
    """Represents auto shut off report."""

    def __init__(
        self,
        timestamp: float,
        type: str,
        duration: int,
        volume: int,
    ) -> None:
        """Create an Auto Shut Off Report object."""
        self.timestamp = timestamp
        self.type = type
        self.duration = duration
        self.volume = volume

    @classmethod
    def from_dict(cls, data: dict):
        """Create an Auto Shut Off Report object from dictionary."""
        return cls(
            timestamp=data.get(TIMESTAMP_FIELD),
            type=data.get(TYPE_FIELD),
            duration=data.get(DURATION_FIELD),
            volume=data.get(VOLUME_FIELD)
        )


from .water_event import Event

FLOW_FIELD = "flow"
PRESSURE_FIELD = "pressure"
TEMPERATURE_FIELD = "temperature"
EVENT_FIELD = "event"
ERRORS_FIELD = "errors"

class TelemetryData:
    """Represents telemetry data."""

    def __init__(
        self,
        flow: float,
        pressure: float,
        water_temperature: float,
        ongoing_event: Event,
        errors: list[str],
    ) -> None:
        """Create an Telemetry Data object."""
        self.flow = flow
        self.pressure = pressure
        self.water_temperature = water_temperature
        self.ongoing_event = ongoing_event
        self.errors = errors

    @classmethod
    def from_dict(cls, data: dict):
        """Create an Telemetry Data object from a dictionary."""
        return cls(
            flow=data.get(FLOW_FIELD),
            pressure=data.get(PRESSURE_FIELD),
            water_temperature=data.get(TEMPERATURE_FIELD),
            ongoing_event=Event(**data.get(EVENT_FIELD)),
            errors=data.get(ERRORS_FIELD)
        )


MQTT_CONNECTED_FIELD = "mqttConnected"
WIFI_CONNECTED_FIELD = "wifiConnected"
IP_FIELD = "ip"
GATEWAY_FIELD = "gateway"
SUBNET_FIELD = "subnet"
SSID_FIELD = "ssid"
RSSI_FIELD = "rssi"
WIFI_UPTIME_FIELD = "wifiUpTime"
MQTT_UPTIME_FIELD = "mqttUpTime"

class NetworkingData:
    """Represents networking data."""

    def __init__(
        self,
        mqtt_connected: bool,
        wifi_connected: bool,
        ip: str,
        gateway: str,
        subnet: str,
        ssid: str,
        rssi: str,
        wifi_uptime: str,
        mqtt_uptime: str,
    ) -> None:
        """Create an Networking object."""
        self.mqtt_connected = mqtt_connected
        self.wifi_connected = wifi_connected
        self.ip = ip
        self.gateway = gateway
        self.subnet = subnet
        self.ssid = ssid
        self.rssi = rssi
        self.wifi_uptime = wifi_uptime
        self.mqtt_uptime = mqtt_uptime

    @classmethod
    def from_dict(cls, data: dict):
        """Create an Networking from a dictionary."""
        return cls(
            mqtt_connected=data.get(MQTT_CONNECTED_FIELD),
            wifi_connected=data.get(WIFI_CONNECTED_FIELD),
            ip=data.get(IP_FIELD),
            gateway=data.get(GATEWAY_FIELD),
            subnet=data.get(SUBNET_FIELD),
            ssid=data.get(SSID_FIELD),
            rssi=data.get(RSSI_FIELD),
            wifi_uptime=data.get(WIFI_UPTIME_FIELD),
            mqtt_uptime=data.get(MQTT_UPTIME_FIELD),
        )


from typing import Optional, List, Dict, Union

# Constants for field names
VALVE_STATE_FIELD = "state"
SUPPLY_FIELD = "supply"
IP_FIELD = "ip"
GATEWAY_FIELD = "gateway"
SUBNET_FIELD = "subnet"
SSID_FIELD = "ssid"
RSSI_FIELD = "rssi"
TYPE_FIELD = "type"
VOLUME_FIELD = "volume"
DURATION_FIELD = "duration"
TIMESTAMP_FIELD = "timestamp"
FLOW_FIELD = "flow"
PRESSURE_FIELD = "pressure"
TEMPERATURE_FIELD = "temperature"
EVENT_FIELD = "event"
ERRORS_FIELD = "errors"


class AutoShutOffReportData:
    """Represents data for the auto-shut-off report webhook event."""
    
    def __init__(self, event_type: str, volume: int, duration: int, timestamp: int):
        """
        Initialize the AutoShutOffReportData object.
        
        :param event_type: Type of the auto-shut-off event ("VOLUME_THRESHOLD" or "DURATION_THRESHOLD").
        :param volume: Volume of water in liters when the event occurred.
        :param duration: Duration in minutes when the event occurred.
        :param timestamp: Timestamp in milliseconds of the event occurrence.
        """
        self.type = event_type
        self.volume = volume
        self.duration = duration
        self.timestamp = timestamp

    @classmethod
    def from_dict(cls, data: dict):
        """Create an AutoShutOffReportData object from a dictionary."""
        return cls(
            event_type=data.get(TYPE_FIELD),
            volume=data.get(VOLUME_FIELD),
            duration=data.get(DURATION_FIELD),
            timestamp=data.get(TIMESTAMP_FIELD)
        )


class TelemetryEventData:
    """Represents data for the telemetry webhook event."""
    
    def __init__(
        self,
        flow: Optional[int] = None,
        pressure: Optional[int] = None,
        temperature: Optional[float] = None,
        event: Optional[Dict[str, int]] = None,
        errors: Optional[List[str]] = None,
    ):
        """
        Initialize the TelemetryEventData object.
        
        :param flow: Flow rate in ml/min.
        :param pressure: Pressure in mbar.
        :param temperature: Temperature in °C.
        :param event: Event-specific data containing volume and duration.
        :param errors: List of errors, if any (e.g., "flow", "pressure", "temperature").
        """
        self.flow = flow
        self.pressure = pressure
        self.temperature = temperature
        self.event = event
        self.errors = errors

    @classmethod
    def from_dict(cls, data: dict):
        """Create a TelemetryEventData object from a dictionary."""
        return cls(
            flow=data.get(FLOW_FIELD),
            pressure=data.get(PRESSURE_FIELD),
            temperature=data.get(TEMPERATURE_FIELD),
            event=data.get(EVENT_FIELD),
            errors=data.get(ERRORS_FIELD)
        )


class ValveEventData:
    """Represents data for the valve state change webhook event."""
    
    def __init__(self, state: str):
        """
        Initialize the ValveEventData object.
        
        :param state: State of the valve ("open", "closed", "opening", or "closing").
        """
        self.state = state

    @classmethod
    def from_dict(cls, data: dict):
        """Create a ValveEventData object from a dictionary."""
        return cls(
            state=data.get(VALVE_STATE_FIELD)
        )


class PowerSupplyChangedEventData:
    """Represents data for the power supply change webhook event."""
    
    def __init__(self, supply: str):
        """
        Initialize the PowerSupplyChangedEventData object.
        
        :param supply: Type of power supply ("battery", "external", or "external+battery").
        """
        self.supply = supply

    @classmethod
    def from_dict(cls, data: dict):
        """Create a PowerSupplyChangedEventData object from a dictionary."""
        return cls(
            supply=data.get(SUPPLY_FIELD)
        )


class WifiChangedEventData:
    """Represents data for the WiFi configuration change webhook event."""
    
    def __init__(self, ip: str, gateway: str, subnet: str, ssid: str, rssi: int):
        """
        Initialize the WifiChangedEventData object.
        
        :param ip: IP address of the device.
        :param gateway: Gateway IP address.
        :param subnet: Subnet mask.
        :param ssid: SSID of the connected WiFi network.
        :param rssi: Received Signal Strength Indicator (RSSI) in dBm.
        """
        self.ip = ip
        self.gateway = gateway
        self.subnet = subnet
        self.ssid = ssid
        self.rssi = rssi

    @classmethod
    def from_dict(cls, data: dict):
        """Create a WifiChangedEventData object from a dictionary."""
        return cls(
            ip=data.get(IP_FIELD),
            gateway=data.get(GATEWAY_FIELD),
            subnet=data.get(SUBNET_FIELD),
            ssid=data.get(SSID_FIELD),
            rssi=data.get(RSSI_FIELD)
        )


class OnlineEvent:
    """Represents an online status webhook event with no specific data."""
    
    def __init__(self):
        """
        Initialize the OnlineEvent object.
        
        This event indicates that the device has come online.
        """
        self.type = "online"
    
    @classmethod
    def from_dict(cls, data: dict):
        """Create an OnlineEvent object from a dictionary."""
        return cls()


class WebhookEvent:
    """Base class for all webhook events."""
    
    def __init__(self, event_type: str, data: dict):
        """
        Initialize the WebhookEvent object.
        
        :param event_type: Type of the webhook event.
        :param data: Data payload associated with the event.
        """
        self.type = event_type
        self.data = data

    @classmethod
    def from_dict(cls, data: dict):
        """Create a base WebhookEvent object from a dictionary."""
        return cls(
            event_type=data.get("type"),
            data=data.get("data", {})
        )
    
    @classmethod
    def parse_webhook_event(cls, data: dict) -> Union[
        AutoShutOffReportData, TelemetryEventData, ValveEventData, 
        PowerSupplyChangedEventData, WifiChangedEventData, OnlineEvent
    ]:
        """
        Parse a raw webhook event dictionary and return the appropriate event object.

        :param data: Dictionary containing the raw webhook event data.
        :return: An instance of the appropriate event class.
        :raises ValueError: If the event type is unknown.
        """
        event_type = data.get("type")
        event_data = data.get("data", {})
        
        if event_type == "auto-shut-off-report":
            return AutoShutOffReportData.from_dict(event_data)
        elif event_type == "telemetry":
            return TelemetryEventData.from_dict(event_data)
        elif event_type == "valve":
            return ValveEventData.from_dict(event_data)
        elif event_type == "power-supply-changed":
            return PowerSupplyChangedEventData.from_dict(event_data)
        elif event_type == "wifi-changed":
            return WifiChangedEventData.from_dict(event_data)
        elif event_type == "online":
            return OnlineEvent.from_dict({})
        else:
            raise ValueError(f"Unknown event type: {event_type}")



from .water_meter import WaterMeter

VALVE_STATE_FIELD = "valveState"
WATER_FLOW_INDICATOR_FIELD = "waterFlowing"
MQTT_STATUS_FIELD = "mqttConnected"
WIFI_STATUS_FIELD = "wifiConnected"
POWER_SUPPLY_FIELD = "powerSupply"
FIRMWARE_VERSION_FIELD = "firmwareVersion"
UPTIME_FIELD = "uptime"
WATER_METER_FIELD = "waterMeter"

class DeviceState:
    """Represents the device state."""

    def __init__(
        self,
        valve_state: str,
        water_flow_indicator: str,
        mqtt_status: str,
        wifi_status: str,
        power_supply: str,
        firmware_version: str,
        uptime: int,
        water_meter: WaterMeter,
    ) -> None:
        """Create an Device State object."""

        self.valve_state = valve_state
        self.water_flow_indicator = water_flow_indicator
        self.mqtt_status = mqtt_status
        self.wifi_status = wifi_status
        self.power_supply = power_supply
        self.firmware_version = firmware_version
        self.uptime = uptime
        self.water_meter = water_meter

    @classmethod
    def from_dict(cls, data: dict):
        """Create an Device State object from a dictionary."""
        return cls(
            valve_state=data.get(VALVE_STATE_FIELD),
            water_flow_indicator=data.get(WATER_FLOW_INDICATOR_FIELD),
            mqtt_status=data.get(MQTT_STATUS_FIELD),
            wifi_status=data.get(WIFI_STATUS_FIELD),
            power_supply=data.get(POWER_SUPPLY_FIELD),
            firmware_version=data.get(FIRMWARE_VERSION_FIELD),
            uptime=data.get(UPTIME_FIELD),
            water_meter=WaterMeter(**(data.get(WATER_METER_FIELD) or {}))
        )


from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="watergate_local_api", # Replace with your own username
    version="2024.1.0",
    author="Watergate",
    author_email="hi@watergate.ai",
    description="Python package to interact with the Watergate Local API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hero-laboratories/watergate-local-api-python",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ]
)


ENABLED_FIELD = "enabled"
VOLUME_THRESHOLD_FIELD = "volumeThreshold"
DURATION_THRESHOLD_FIELD = "durationThreshold"

class AutoShutOffState:
    """Represents Auto Shut Off state."""

    def __init__(
        self,
        enabled: bool,
        volume_threshold: int,
        duration_threshold: int,
    ) -> None:
        """Create an Auto Shut Off object."""
        self.enabled = enabled
        self.volume_threshold = volume_threshold
        self.duration_threshold = duration_threshold

    @classmethod
    def from_dict(cls, data: dict):
        """Create an Auto Shut Off object from a dictionary."""
        return cls(
            enabled=data.get(ENABLED_FIELD),
            volume_threshold=data.get(VOLUME_THRESHOLD_FIELD),
            duration_threshold=data.get(DURATION_THRESHOLD_FIELD),
        )


import logging
from typing import Optional

import aiohttp
import asyncio

from .models import (
    DeviceState, 
    NetworkingData,
    TelemetryData,
    AutoShutOffState,
    AutoShutOffReport
)

_LOGGER = logging.getLogger(__name__)

ACCEPT_HEADER = "Accept"
CONTENT_TYPE_HEADER = "Content-Type"

NETWORKING_URL = "/networking"
TELEMETRY_URL = "/telemetry"
AUTO_SHUT_OFF_URL = "/auto-shut-off"
AUTO_SHUT_OFF_REPORT_URL = "/auto-shut-off/report"
WEBHOOK_URL = "/webhook"

RETRY_ATTEMPTS = range(3)

class WatergateApiException(Exception):
    """Custom exception for critical errors in WatergateLocalApiClient."""
    pass

class WatergateLocalApiClient:
    """API Client for interacting with the external service."""

    def __init__(self, base_url: str, timeout: int = 10) -> None:
        """Initialize the API client."""
        self._base_url = base_url + "/api/sonic"
        self._timeout = aiohttp.ClientTimeout(total=timeout)
        self._session: Optional[aiohttp.ClientSession] = None

    async def __aenter__(self):
        self._session = aiohttp.ClientSession(timeout=self._timeout)
        return self

    async def __aexit__(self, exc_type, exc, tb):
        if self._session:
            await self._session.close()

    async def _get(self, url: str, headers: dict) -> Optional[dict]:
        """Helper method to perform GET requests."""
        if not self._session:
            raise RuntimeError("Session not initialized")

        for attempt in RETRY_ATTEMPTS:  # Retry logic
            try:
                async with self._session.get(url, headers=headers) as response:
                    if response.status == 200:
                        return await response.json()
                    _LOGGER.error("Failed to fetch data from %s: %s", url, response.status)
            except aiohttp.ClientError as e:
                _LOGGER.error("Network error occurred: %s", e)
            await asyncio.sleep(1)  # Wait before retrying
        raise WatergateApiException(f"Failed to fetch data from {url} after 3 attempts")

    async def async_get_device_state(self) -> Optional[DeviceState]:
        """GET /api/sonic/ - Get device state."""
        headers = {ACCEPT_HEADER: "application/vnd.wtg.local.device-state.v1+json"}
        data = await self._get(self._base_url, headers)
        return DeviceState.from_dict(data) if data else None

    async def async_get_networking(self) -> Optional[NetworkingData]:
        """GET /api/sonic/networking - Get networking."""
        url = self._base_url + NETWORKING_URL
        headers = {ACCEPT_HEADER: "application/vnd.wtg.local.networking.v1+json"}
        data = await self._get(url, headers)
        return NetworkingData.from_dict(data) if data else None

    async def async_get_telemetry_data(self) -> Optional[TelemetryData]:
        """GET /api/sonic/telemetry - Get telemetry data."""
        url = self._base_url + TELEMETRY_URL
        headers = {ACCEPT_HEADER: "application/vnd.wtg.local.telemetry.v1+json"}
        data = await self._get(url, headers)
        return TelemetryData.from_dict(data) if data else None

    async def async_patch_auto_shut_off(
        self, enabled: Optional[bool] = None, duration: Optional[int] = None, volume: Optional[int] = None
    ) -> bool:
        """PATCH /api/sonic/auto-shut-off - Patch auto shut off."""
        url = self._base_url + AUTO_SHUT_OFF_URL
        headers = {CONTENT_TYPE_HEADER: "application/vnd.wtg.local.auto-shut-off.v1+json"}

        data = {}

        if enabled is not None:
            data["enabled"] = enabled
        if duration is not None:
            data["durationThreshold"] = duration
        if volume is not None:
            data["volumeThreshold"] = volume

        for attempt in RETRY_ATTEMPTS:  # Retry logic
            try:
                async with self._session.patch(url, json=data, headers=headers) as response:
                    if response.status == 204:
                        return True
                    _LOGGER.error("Failed to set auto shut off parameter: %s", response.status)
            except aiohttp.ClientError as e:
                _LOGGER.error("Network error occurred: %s", e)
            await asyncio.sleep(1)  # Wait before retrying
        raise WatergateApiException(f"Failed to patch auto shut off after 3 attempts")

    async def async_get_auto_shut_off_report(self) -> Optional[AutoShutOffReport]:
        """GET /api/sonic/auto-shut-off/report - Get auto shut-off report."""
        url = self._base_url + AUTO_SHUT_OFF_REPORT_URL
        headers = {ACCEPT_HEADER: "application/vnd.wtg.local.auto-shut-off.report.v1+json"}

        data = await self._get(url, headers)
        return AutoShutOffReport.from_dict(data) if data else None

    async def async_set_webhook_url(self, webhook: str) -> bool:
        """PATCH /api/sonic/webhook - Set webhook URL."""
        url = self._base_url + WEBHOOK_URL
        headers = {CONTENT_TYPE_HEADER: "application/vnd.wtg.local.webhook.v1+json"}
        data = {"webhookUrl": webhook}

        for attempt in RETRY_ATTEMPTS:  # Retry logic
            try:
                async with self._session.patch(url, json=data, headers=headers) as response:
                    if response.status == 204:
                        return True
                    _LOGGER.error("Failed to set webhook URL: %s", response.status)
            except aiohttp.ClientError as e:
                _LOGGER.error("Network error occurred: %s", e)
            await asyncio.sleep(1)  # Wait before retrying
        raise WatergateApiException(f"Failed to set webhook URL after 3 attempts")


from .device_state import DeviceState
from .networking import NetworkingData
from .telemetry_data import TelemetryData
from .auto_shut_off_state import AutoShutOffState
from .auto_shut_off_report import AutoShutOffReport
